@@ -5,6 +5,7 @@ use crate::config::colors::ColorRgb;
 use crate::crosswords::grid::Scroll;
 use crate::crosswords::pos::{Direction, Pos};
 use crate::crosswords::search::{Match, RegexSearch};
+use crate::crosswords::LineDamage;
 use crate::error::RioError;
 use rio_window::event::Event as RioWindowEvent;
 use std::borrow::Cow;
@@ -44,14 +45,33 @@ pub enum ClickState {
     TripleClick,
 }
 
+/// Terminal damage information for efficient rendering
+#[derive(Debug, Clone)]
+pub enum TerminalDamage {
+    /// The entire terminal needs to be redrawn
+    Full,
+    /// Only specific lines need to be redrawn
+    Partial(Vec<LineDamage>),
+    /// Only the cursor position has changed
+    CursorOnly,
+}
+
 #[derive(Clone)]
 pub enum RioEvent {
     PrepareRender(u64),
     PrepareRenderOnRoute(u64, usize),
+    PrepareUpdateConfig,
     /// New terminal content available.
     Render,
     /// New terminal content available per route.
     RenderRoute(usize),
+    /// Wake up and check for terminal updates.
+    Wakeup(usize),
+    /// Terminal content has changed with specific damage information.
+    TerminalDamaged {
+        route_id: usize,
+        damage: TerminalDamage,
+    },
     Paste,
     Copy(String),
     UpdateFontSize(u8),
@@ -130,7 +150,8 @@ pub enum RioEvent {
 
     BlinkCursor(u64, usize),
 
-    UpdateGraphicLibrary,
+    /// Update window titles.
+    UpdateTitles,
 
     // No operation
     Noop,
@@ -159,12 +180,29 @@ impl Debug for RioEvent {
             }
             RioEvent::MouseCursorDirty => write!(f, "MouseCursorDirty"),
             RioEvent::ResetTitle => write!(f, "ResetTitle"),
+            RioEvent::PrepareUpdateConfig => write!(f, "PrepareUpdateConfig"),
             RioEvent::PrepareRender(millis) => write!(f, "PrepareRender({millis})"),
             RioEvent::PrepareRenderOnRoute(millis, route) => {
                 write!(f, "PrepareRender({millis} on route {route})")
             }
             RioEvent::Render => write!(f, "Render"),
             RioEvent::RenderRoute(route) => write!(f, "Render route {route}"),
+            RioEvent::Wakeup(route) => {
+                write!(f, "Wakeup route {route}")
+            }
+            RioEvent::TerminalDamaged { route_id, damage } => match damage {
+                TerminalDamage::Full => {
+                    write!(f, "TerminalDamaged route {route_id} (Full)")
+                }
+                TerminalDamage::Partial(lines) => write!(
+                    f,
+                    "TerminalDamaged route {route_id} ({} lines)",
+                    lines.len()
+                ),
+                TerminalDamage::CursorOnly => {
+                    write!(f, "TerminalDamaged route {route_id} (CursorOnly)")
+                }
+            },
             RioEvent::Scroll(scroll) => write!(f, "Scroll {scroll:?}"),
             RioEvent::Bell => write!(f, "Bell"),
             RioEvent::Exit => write!(f, "Exit"),
@@ -188,11 +226,11 @@ impl Debug for RioEvent {
             RioEvent::BlinkCursor(timeout, route_id) => {
                 write!(f, "BlinkCursor {timeout} {route_id}")
             }
+            RioEvent::UpdateTitles => write!(f, "UpdateTitles"),
             RioEvent::Noop => write!(f, "Noop"),
             RioEvent::Copy(_) => write!(f, "Copy"),
             RioEvent::Paste => write!(f, "Paste"),
             RioEvent::UpdateFontSize(action) => write!(f, "UpdateFontSize({action:?})"),
-            RioEvent::UpdateGraphicLibrary => write!(f, "UpdateGraphicLibrary"),
         }
     }
 }
@@ -220,6 +258,18 @@ pub trait OnResize {
     fn on_resize(&mut self, window_size: WinsizeBuilder);
 }
 
+/// Event emitter trait for terminal components
+pub trait TerminalEventEmitter {
+    /// Emit a terminal damage event
+    fn emit_damage(&self, route_id: usize, damage: TerminalDamage);
+
+    /// Emit a cursor change event
+    fn emit_cursor_change(&self, route_id: usize);
+
+    /// Emit a full redraw event
+    fn emit_full_redraw(&self, route_id: usize);
+}
+
 /// Event Loop for notifying the renderer about terminal events.
 pub trait EventListener {
     fn event(&self) -> (Option<RioEvent>, bool);
@@ -273,6 +323,28 @@ impl EventListener for EventProxy {
     }
 }
 
+impl TerminalEventEmitter for EventProxy {
+    fn emit_damage(&self, route_id: usize, damage: TerminalDamage) {
+        // For now, we'll send to the first available window
+        // In a real implementation, you'd need to map route_id to window_id
+        let _window_id = WindowId::from(0); // This needs proper mapping
+        self.send_event(
+            RioEventType::Rio(RioEvent::TerminalDamaged { route_id, damage }),
+            _window_id,
+        );
+    }
+
+    fn emit_cursor_change(&self, route_id: usize) {
+        let _window_id = WindowId::from(0); // This needs proper mapping
+        self.emit_damage(route_id, TerminalDamage::CursorOnly);
+    }
+
+    fn emit_full_redraw(&self, route_id: usize) {
+        let _window_id = WindowId::from(0); // This needs proper mapping
+        self.emit_damage(route_id, TerminalDamage::Full);
+    }
+}
+
 /// Regex search state.
 pub struct SearchState {
     /// Search direction.
